
// Interprets each message and makes changes to the database.
// Checks that the message format is ok. Does not check content for correctness.
class ttchess_interpreter extends ttchess_common {

	ttchess_database db;

	public ttchess_interpreter(ttchess_database _db) {
		db = _db;
	}

	// Interpreter for one message.
	public boolean interpret(int[] data) {

		int messagetype = data[0];
		int len = ((int)data[1] * 0x80) + (int)data[2];
		int address = ((int)data[3] * 0x80) + (int)data[4];

		if (check_checksum(data)==false) {
			System.out.println("Message failed checksum test");
			return false;
		}

		if (data[0] == DGT_BUS_PING) {
			if (data.length != 6) {  // address+data
				System.out.println("Illegal: Got a BUS_PING with wrong size");
				return false;
			}
			db.get_board(address).set_alive(true); // will create a new board if it does not exist
			return true;
		}

		else if (data[0] == DGT_BUS_BRD_DUMP) {
			if (data.length != (6+64)) {
				System.out.println("Illegal: Got a BUS_BRD_DUMP with wrong size");
				return false;
			}
			ttchess_chessboard c = db.get_board(address);
			c.set_boarddump(data, 5);
			return true;
		}

		else if (data[0] == DGT_BUS_UPDATE) {

			ttchess_chessboard c = db.get_board(address);

			for (int i=5; i<data.length-1; i=i+1) {
				if (data[i]==EE_POWERUP) {} // 1 byte 
				else if (data[i]==EE_EOF) {} // 1 byte, but I don't think it will happen 
				else if (data[i]==EE_FOURROWS) {} // 1 byte
				else if (data[i]==EE_EMPTYBOARD) {
					c.set_emptyboard();
				}
				else if (data[i]==EE_DOWNLOADED) {} // 1 byte
				else if (data[i]==EE_BEGINPOS) {} // 1 byte
				else if (data[i]==EE_BEGINPOS_ROT) {} // 1 byte
				else if (data[i]==EE_START_TAG) {} // 1 byte
				else if (data[i]==EE_WATCHDOG_ACTION) {} // 1 byte
				else if (data[i]==EE_NOP) {} // 1 byte
				else if (data[i]>=0x40 && data[i]<=0x5f) { // 2-byte piece update
					c.set_fieldupdate(data[i] & 0x0F, data[i+1]);  // piece,pos
					i=i+1;
				}
				else if ((data[i]>=0x60 && data[i]<=0x69) || (data[i]>=0x70 && data[i]<=0x79)){
					// 3-byte time message
					i=i+2;
				}
				else {
					System.out.println("Unknown data in DGT_BUS_UPDATE command.");
					return false;
				}
			}
			c.set_updated(true);
			return true;
		}

		else if (data[0] == DGT_BUS_BWTIME) {}
		else if (data[0] == DGT_BUS_FROM_START) {}
		else if (data[0] == DGT_BUS_UPDATE) {}
		else if (data[0] == DGT_BUS_START_GAME_WRITTEN) {}
		else if (data[0] == DGT_BUS_VERSION) {}

		System.err.println("Unknown message (checksum was ok).");
		return true;  // It probably is a correct message, just unknown to me.
	}

	boolean check_checksum(int[] data) {
		int sum = 0;
		for (int i=0; i<(data.length-1); i=i+1) sum = (sum + data[i]) & 0x7F;
		if (sum == data[data.length-1]) return true;
		return false;
	}
}
