
// Splits incoming data into separate messages and calls the interpreter with
// each message, and throws away anything unknown.
class ttchess_receiver extends ttchess_common {

	int messagetype;
	int messagelen_msb;
	int messagelen_lsb;
	int[] data;  // The whole message
	int useddata;  // how much of data[] that is currently filled
	int state;  // 0=start, 1=messagetype set, 2=len_msb set, 3=len_lsb set

	ttchess_interpreter interpreter;

	public ttchess_receiver(ttchess_interpreter _interpreter) {
		state = 0;
		interpreter = _interpreter;  // Interprets complete messages
	}

	// The caller should not have to specify startindex=0
	void receive(int[] d) {

		// Print the message, for debugging
		char c[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
		StringBuffer x = new StringBuffer("Received: ");
		for (int i=0; i<d.length; i=i+1) {
			x.append(c[d[i]>>4]).append(c[d[i] & 0x0F]).append(" ");
		}
		if (ttchess.debug) System.out.println(x.toString());

		_receive (d, 0);
	}

	// Receives data and sends each whole message to the messagereceiver. Slightly recursive.
	// Stores non-complete data between successive calls.
	void _receive(int[] d, int start) {

		if (start == d.length) return;  // No more bytes

		if (state == 0) {  // Find a known messageid.
			while (start<d.length) {
				if (is_busmessage(d[start])) {
					messagetype = d[start];
					state = 1;
					_receive(d,start+1);  // interpret next byte, if any.
					return;
				}
				System.err.println("Did not understand 1 byte of incoming data.");
				start = start + 1;  // skip this byte
			}
			return; // nothing in the buffer was understood
		}

		if (state == 1) { // messagelen msb. Bit 8 must not be set.
			if ((d[start] & 0x80) > 0) {  // Bit 8 is set. That is wrong. Restart.
				System.err.println("Did not understand 2 bytes of incoming data.");
				state = 0;
				_receive(d, start);
				return;
			}
			messagelen_msb = d[start];
			state = 2;
			start = start + 1;
		}

		if (start == d.length) return;  // No more bytes

		if (state == 2) { // messagelen lsb. Bit 8 must not be set.
			if ((d[start] & 0x80) > 0) {  // Bit 8 is set. That is wrong. Restart.
				System.err.println("Did not understand 3 bytes of incoming data.");
				state = 0;
				_receive(d, start);
				return;
			}
			messagelen_lsb = d[start];
			state = 3;
			start = start + 1;

			int newlen = ((int)messagelen_msb * 0x80) + (int)messagelen_lsb;
			if (newlen <5) {
				System.out.println("Too small message length: " + Integer.toString(newlen));
				state = 0;
				_receive(d, start);
				return;
			}
			data = new int[newlen];  // Allocate for whole message
			data[0] = messagetype;  // Copy already received bytes
			data[1] = messagelen_msb;
			data[2] = messagelen_lsb;
			useddata = 3;
		}

		if (start == d.length) return;  // No more bytes

		if (state == 3) {  // messageheader ok, now the messagedata
			while (start<d.length && useddata<data.length) { // fill the buffer
				data[useddata]=d[start];  // not so ultra efficient array copying
				start = start + 1;
				useddata = useddata + 1;
			}
			if (useddata == data.length) { // Yes! We have a complete message. Maybe.
				// Here, if message was not correct, we might want to push everything
				// except the first byte back to the incoming data queue. That way,
				// we really would try very hard to find a message.
				//if (interpreter.interpret(data)==false) { ...not done yet... }
				interpreter.interpret(data);

				start = start + 1;
				data = null;
				state = 0;  // start of a new message
			}
		}
		_receive(d, start);  // loop until there is no data to process
	}
}
